@@ -66,6 +66,31 @@ static siod_LISP option_names_table = NIL;
 #define NUMBER_BUF_SIZE 64
 static char number_buf[NUMBER_BUF_SIZE];
 
+static siod_LISP
+muesli_to_siod(muesli_value_t mval)
+{
+  switch (mval.type) {
+  case muesli_value_none:
+    return NIL;
+  case muesli_value_float:
+    return siod_flocons(mval.data.as_float);
+  case muesli_value_integer:
+    return siod_flocons(mval.data.as_int);
+  case muesli_value_string:
+    return siod_strcons(strlen(mval.data.as_string), mval.data.as_string);
+  case muesli_value_boolean:
+    if (mval.data.as_int) {
+      return sym_t;
+    } else {
+      return NIL;
+    }
+  case muesli_value_error:
+    // todo: should probably throw
+    return NIL;
+  }
+    return NIL;
+}
+
 static siod_LISP
 muesli_siod_set_parameter(siod_LISP option, siod_LISP value)
 {
@@ -107,28 +132,12 @@ muesli_siod_get_parameter(siod_LISP option)
   char option_code = muesli_find_option_letter(current_interface->getopt_options, get_c_string(option));
 
   if (option_code != -1) {
-    muesli_value_t result = (current_interface->handle_option)(current_interface->app_params,
-							       option_code,	// option
-							       NULL, 0.0,	// value
-							       0,	// set
-							       (const char *)"siod");
+    return muesli_to_siod((current_interface->handle_option)(current_interface->app_params,
+							     option_code,	// option
+							     NULL, 0.0,	// value
+							     0,	// set
+							     (const char *)"siod"));
 
-    switch (result.type) {
-    case muesli_value_string:
-      return siod_strcons(strlen(result.data.as_string), result.data.as_string);
-    case muesli_value_float:
-      return siod_flocons(result.data.as_float);
-    case muesli_value_integer:
-      return siod_flocons((float)result.data.as_int);
-    case muesli_value_boolean:
-      if (result.data.as_int) {
-	return sym_t;
-      } else {
-	return NIL;
-      }
-    default:
-      return NIL;
-    }
   } else {
     return NIL;
   }
@@ -283,30 +292,11 @@ muesli_siod_eval_in_language(siod_LISP language_name, siod_LISP evaluand)
 
     // fprintf(stderr, "In muesli_siod_eval_in_language(\"%s\", \"%s\")\n", language, scratch);
 
-    muesli_value_t result = muesli_eval_in_language(language,
-						    scratch,
-						    string_length,
-						    ambient_transient);
+    return muesli_to_siod(muesli_eval_in_language(language,
+						  scratch,
+						  string_length,
+						  ambient_transient));
 
-    switch (result.type) {
-    case muesli_value_none:
-      return NIL;
-    case muesli_value_float:
-      return siod_flocons(result.data.as_float);
-    case muesli_value_integer:
-      return siod_flocons(result.data.as_int);
-    case muesli_value_string:
-      return siod_strcons(strlen(result.data.as_string), result.data.as_string);
-    case muesli_value_boolean:
-      if (result.data.as_int) {
-	return sym_t;
-      } else {
-	return NIL;
-      }
-    case muesli_value_error:
-      // todo: should probably throw
-      return NIL;
-    }   
   } else {
 #if 0
     // todo: make this a throw, or something like that
