@@ -71,6 +71,11 @@ muesli_make_evaluator_interface(const char* new_name,
   interface->specializer = NULL;
   interface->close_evaluator = muesli_default_close_evaluator;
   interface->add_function = NULL;
+  interface->eval_string = NULL;
+  interface->load_file = NULL;
+  interface->to_muesli_value = NULL;
+  interface->from_muesli_value = NULL;
+  
   interface->handle_option = new_option_handler;
   interface->getopt_options = new_getopt_options;
   interface->repl = NULL;
@@ -187,18 +192,52 @@ evaluator_interface *
 muesli_get_evaluator_by_extension(const char *extension)
 {
   evaluator_interface *evaluator;
+
+  evaluator_interface *choice_1 = NULL;
+  evaluator_interface *choice_2 = NULL;
+
   for (evaluator = evaluator_interfaces;
        evaluator != NULL;
        evaluator = evaluator->next) {
     if ((evaluator->extension != NULL)
 	&& strcmp(evaluator->extension, extension) == 0) {
-
-      // Make sure it's ready for use
-      muesli_initialize_evaluator(evaluator);
-
-      return evaluator;
+      if (choice_1 == NULL) {
+	fprintf(stderr, "First possibility for %s is %s\n", extension, evaluator->name);
+	choice_1 = evaluator;
+      } else  {
+	fprintf(stderr, "Other possibility for %s is %s\n", extension, evaluator->name);
+	choice_2 = evaluator;
+      }
     }
   }
+
+  if ((choice_1 != NULL)) {
+    /* If we got more than one evaluating offering to run this file,
+       look to see whether the user has expressed a preference: */
+    if (choice_2 != NULL) {
+      char *preferences = getenv("MUESLI_PREFERENCES");
+      if (preferences != NULL) {
+	char *name_2 = choice_2->name;
+	unsigned int name_2_len = strlen(name_2);
+
+
+	for (;
+	     preferences != NULL;
+	     preferences = strchr(preferences, ',')) {
+	  fprintf(stderr, "Looking in \"%s\" for \"%s\"\n", preferences, name_2);
+	  if (strncmp(preferences, name_2, name_2_len) == 0) {
+	    choice_1 = choice_2;
+	    break;
+	  }
+	}
+      }
+    }
+
+    // Make sure it's ready for use
+    muesli_initialize_evaluator(choice_1);
+
+    return choice_1;
+  }
   return NULL;
 }
 
@@ -351,27 +390,30 @@ evaluator_interface *no_evaluator = NULL; // might put in a novelty one to print
 
 evaluator_interface *scripting_evaluator = NULL;
 
-evaluator_interface *evaluator_zero = NULL;
-evaluator_interface *evaluator_siod = NULL;
-evaluator_interface *evaluator_lua = NULL;
+evaluator_interface *evaluator_R = NULL;
+evaluator_interface *evaluator_common_lisp = NULL;
+evaluator_interface *evaluator_custom = NULL;
+evaluator_interface *evaluator_erlang = NULL;
 evaluator_interface *evaluator_external_exec = NULL;
 evaluator_interface *evaluator_external_pipe = NULL;
-evaluator_interface *evaluator_custom = NULL;
-evaluator_interface *evaluator_perl = NULL;
-evaluator_interface *evaluator_tcl = NULL;
-evaluator_interface *evaluator_python = NULL;
-evaluator_interface *evaluator_tcc = NULL;
-evaluator_interface *evaluator_octave = NULL;
-evaluator_interface *evaluator_slang = NULL;
-evaluator_interface *evaluator_ruby = NULL;
-evaluator_interface *evaluator_prolog = NULL;
-evaluator_interface *evaluator_parrot = NULL;
-evaluator_interface *evaluator_jvm = NULL;
-evaluator_interface *evaluator_javascript = NULL;
-evaluator_interface *evaluator_machine = NULL;
-evaluator_interface *evaluator_stack_code = NULL;
-evaluator_interface *evaluator_common_lisp = NULL;
+evaluator_interface *evaluator_guile = NULL;
 evaluator_interface *evaluator_haskell = NULL;
+evaluator_interface *evaluator_javascript = NULL;
+evaluator_interface *evaluator_jvm = NULL;
+evaluator_interface *evaluator_lua = NULL;
+evaluator_interface *evaluator_machine = NULL;
+evaluator_interface *evaluator_octave = NULL;
+evaluator_interface *evaluator_parrot = NULL;
+evaluator_interface *evaluator_perl = NULL;
+evaluator_interface *evaluator_prolog = NULL;
+evaluator_interface *evaluator_python = NULL;
+evaluator_interface *evaluator_ruby = NULL;
+evaluator_interface *evaluator_siod = NULL;
+evaluator_interface *evaluator_slang = NULL;
+evaluator_interface *evaluator_stack_code = NULL;
+evaluator_interface *evaluator_tcc = NULL;
+evaluator_interface *evaluator_tcl = NULL;
+evaluator_interface *evaluator_zero = NULL;
 
 /* You should use set_evaluator_app_specializer(string,
    specializer_function) to set a function to add your extensions.
@@ -491,11 +533,35 @@ muesli_register_evaluators(void *new_app_params,
 					   zero_setup, zero_evaluator_init,
 					   params_handler, getopt_options,
 					   new_app_params, new_app_data);
-#if defined(HAVE_LIBSIOD) || defined(HAVE_LIBULSIOD)
-  evaluator_siod = muesli_define_evaluator("siod", "scm", 0,
-					   siod_setup, siod_evaluator_init,
-					   params_handler, getopt_options,
-					   new_app_params, new_app_data);
+#ifdef HAVE_LIBR
+  evaluator_R = muesli_define_evaluator("R", "r", 0,
+					R_setup, R_evaluator_init,
+					params_handler, getopt_options,
+					new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBERL_INTERFACE
+  evaluator_erlang = muesli_define_evaluator("erlang", "er", 0,
+					     erlang_setup, erlang_evaluator_init,
+					     params_handler, getopt_options,
+					     new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBGUILE
+  evaluator_guile = muesli_define_evaluator("guile", "scm", 0,
+					    guile_setup, guile_evaluator_init,
+					    params_handler, getopt_options,
+					    new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBMOZJS
+  evaluator_javascript = muesli_define_evaluator("javascript", "js", 0,
+						 javascript_setup, javascript_evaluator_init,
+						 params_handler, getopt_options,
+						 new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBJVM
+  evaluator_jvm = muesli_define_evaluator("jvm", "class", 1,
+					  jvm_setup, jvm_evaluator_init,
+					  params_handler, getopt_options,
+					  new_app_params, new_app_data);
 #endif
 #ifdef HAVE_LIBLUA
   evaluator_lua = muesli_define_evaluator("lua", "lua", 0,
@@ -503,69 +569,63 @@ muesli_register_evaluators(void *new_app_params,
 					  params_handler, getopt_options,
 					  new_app_params, new_app_data);
 #endif
-#ifdef HAVE_LIBPERL
-  evaluator_perl = muesli_define_evaluator("perl", "pl", 0,
-					   perl_setup, perl_evaluator_init,
-					   params_handler, getopt_options,
-					   new_app_params, new_app_data);
-#endif
-#ifdef HAVE_LIBTCL
-  evaluator_tcl = muesli_define_evaluator("tcl", "tcl", 0,
-					  tcl_setup, tcl_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
-#endif
-#ifdef HAVE_LIBPYTHON
-  evaluator_python = muesli_define_evaluator("python", "python", 0,
-					     python_setup, python_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
-#endif
-#ifdef HAVE_LIBTCC
-  evaluator_tcc = muesli_define_evaluator("tcc", "c", 0,
-					  tcc_setup, tcc_evaluator_init,
-					  params_handler, getopt_options,
-					  new_app_params, new_app_data);
-#endif
 #ifdef HAVE_LIBOCTINTERP
   evaluator_octave = muesli_define_evaluator("octave", "oct", 0,
 					     octave_setup, octave_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
-#ifdef HAVE_LIBSLANG
-  evaluator_slang = muesli_define_evaluator("s-lang", "sl", 0,
-					    slang_setup, slang_evaluator_init,
-					    params_handler, getopt_options,
-					    new_app_params, new_app_data);
-#endif
-#ifdef HAVE_LIBRUBY
-  evaluator_ruby = muesli_define_evaluator("ruby", "ruby", 0,
-					   ruby_setup, ruby_evaluator_init,
-					   params_handler, getopt_options,
-					   new_app_params, new_app_data);
-#endif
-#ifdef HAVE_LIBPROLOG
-  evaluator_prolog = muesli_define_evaluator("prolog", "pro", 0,
-					     prolog_setup, prolog_evaluator_init,
-					     params_handler, getopt_options,
-					     new_app_params, new_app_data);
-#endif
-#ifdef HAVE_LIBMOZJS
-  evaluator_javascript = muesli_define_evaluator("javascript", "js", 0,
-						 javascript_setup, javascript_evaluator_init,
-						 params_handler, getopt_options,
-						 new_app_params, new_app_data);
-#endif
 #ifdef HAVE_LIBPARROT
   evaluator_parrot = muesli_define_evaluator("parrot", "peg", 1,
 					     parrot_setup, parrot_evaluator_init,
 					     params_handler, getopt_options,
 					     new_app_params, new_app_data);
 #endif
-#ifdef HAVE_LIBJVM
-  evaluator_jvm = muesli_define_evaluator("jvm", "class", 1,
-					  jvm_setup, jvm_evaluator_init,
+#ifdef HAVE_LIBPERL
+  evaluator_perl = muesli_define_evaluator("perl", "pl", 0,
+					   perl_setup, perl_evaluator_init,
+					   params_handler, getopt_options,
+					   new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBPL
+  evaluator_prolog = muesli_define_evaluator("prolog", ".pl", 0,
+					     prolog_setup, prolog_evaluator_init,
+					     params_handler, getopt_options,
+					     new_app_params, new_app_data);
+#endif
+#if defined(HAVE_LIBPYTHON) && defined(HAVE_PYTHON2_6_PYTHON_H)
+  evaluator_python = muesli_define_evaluator("python", "python", 0,
+					     python_setup, python_evaluator_init,
+					     params_handler, getopt_options,
+					     new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBRUBY_STATIC
+  evaluator_ruby = muesli_define_evaluator("ruby", "rb", 0,
+					   ruby_setup, ruby_evaluator_init,
+					   params_handler, getopt_options,
+					   new_app_params, new_app_data);
+#endif
+#if defined(HAVE_LIBSIOD) || defined(HAVE_LIBULSIOD)
+  evaluator_siod = muesli_define_evaluator("siod", "scm", 0,
+					   siod_setup, siod_evaluator_init,
+					   params_handler, getopt_options,
+					   new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBSLANG
+  evaluator_slang = muesli_define_evaluator("s-lang", "sl", 0,
+					    slang_setup, slang_evaluator_init,
+					    params_handler, getopt_options,
+					    new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBTCC
+  evaluator_tcc = muesli_define_evaluator("tcc", "c", 0,
+					  tcc_setup, tcc_evaluator_init,
+					  params_handler, getopt_options,
+					  new_app_params, new_app_data);
+#endif
+#ifdef HAVE_LIBTCL
+  evaluator_tcl = muesli_define_evaluator("tcl", "tcl", 0,
+					  tcl_setup, tcl_evaluator_init,
 					  params_handler, getopt_options,
 					  new_app_params, new_app_data);
 #endif
